(* rasclDict.mli -- Copyright  2006 by Matthew C. Gushee 
   This program is free software, released under the terms of the BSD
   license. See the file LICENSE for details. *)

type t  (** The type of dictionaries *)
type data = S of string | L of string list | D of t

val create : ?locked:bool -> unit -> t
  (** Use [create ?locked ()] to create a new dictionary. The [locked]
      parameter indicates whether *)
val get : ?prefix:string list -> string -> t -> data
val set : ?prefix:string list -> string -> data -> t -> unit 
val get_string : ?prefix:string list -> string -> t -> string
val get_dict : ?prefix:string list -> string -> t -> t
val get_list : ?prefix:string list -> string -> t -> string list
val set_string : ?prefix:string list -> string -> string -> t -> unit
val set_list : ?prefix:string list -> string -> string list -> t -> unit
val set_dict : ?prefix:string list -> string -> t -> t -> unit
val copy : t -> t

(** {2 Higher-order functions} *)

val iter : (string -> data -> unit) -> t -> unit
val map : (string -> data -> data) -> t -> t
val filter : (string -> data -> bool) -> t -> t
val do_in : string list -> (t -> unit) list -> t -> unit
val get_from : string list -> string list -> t -> (string * data) list
val set_in : string list -> (string * data) list -> t -> unit

module Unsafe : sig
  (** Everything in this module is subject to change without
    *  notice. Use at your own risk. *)
  val set_size : int -> unit
  val get_subdict : string list -> t -> t
  val detach_subdict : string list -> t -> t
  val alist_of_dict : ?strict:bool -> ?prefix:string list ->
    t -> (string * string) list
  val exists : ?prefix:string list -> (string -> data -> bool) ->
    t -> bool
  val lock_structure : t -> unit
  val unlock_structure : t -> unit
  val update_from : t -> t -> unit
  val dump : t -> unit
end
